from pyarrow.lib import DataType

def is_null(t: DataType) -> bool: ...
def is_boolean(t: DataType) -> bool: ...
def is_integer(t: DataType) -> bool: ...
def is_signed_integer(t: DataType) -> bool: ...
def is_unsigned_integer(t: DataType) -> bool: ...
def is_int8(t: DataType) -> bool: ...
def is_int16(t: DataType) -> bool: ...
def is_int32(t: DataType) -> bool: ...
def is_int64(t: DataType) -> bool: ...
def is_uint8(t: DataType) -> bool: ...
def is_uint16(t: DataType) -> bool: ...
def is_uint32(t: DataType) -> bool: ...
def is_uint64(t: DataType) -> bool: ...
def is_floating(t: DataType) -> bool: ...
def is_float16(t: DataType) -> bool: ...
def is_float32(t: DataType) -> bool: ...
def is_float64(t: DataType) -> bool: ...
def is_list(t: DataType) -> bool: ...
def is_large_list(t: DataType) -> bool: ...
def is_fixed_size_list(t: DataType) -> bool: ...
def is_list_view(t: DataType) -> bool: ...
def is_large_list_view(t: DataType) -> bool: ...
def is_struct(t: DataType) -> bool: ...
def is_union(t: DataType) -> bool: ...
def is_nested(t: DataType) -> bool: ...
def is_run_end_encoded(t: DataType) -> bool: ...
def is_temporal(t: DataType) -> bool: ...
def is_timestamp(t: DataType) -> bool: ...
def is_duration(t: DataType) -> bool: ...
def is_time(t: DataType) -> bool: ...
def is_time32(t: DataType) -> bool: ...
def is_time64(t: DataType) -> bool: ...
def is_binary(t: DataType) -> bool: ...
def is_large_binary(t: DataType) -> bool: ...
def is_unicode(t: DataType) -> bool: ...
def is_string(t: DataType) -> bool: ...
def is_large_unicode(t: DataType) -> bool: ...
def is_large_string(t: DataType) -> bool: ...
def is_fixed_size_binary(t: DataType) -> bool: ...
def is_binary_view(t: DataType) -> bool: ...
def is_string_view(t: DataType) -> bool: ...
def is_date(t: DataType) -> bool: ...
def is_date32(t: DataType) -> bool: ...
def is_date64(t: DataType) -> bool: ...
def is_map(t: DataType) -> bool: ...
def is_decimal(t: DataType) -> bool: ...
def is_decimal128(t: DataType) -> bool: ...
def is_decimal256(t: DataType) -> bool: ...
def is_dictionary(t: DataType) -> bool: ...
def is_interval(t: DataType) -> bool: ...
def is_primitive(t: DataType) -> bool: ...
